/*
 * ar3cmonitor - analogcache.h
 *
 * 	Analog Cache
 *
 * Copyright (C) 2007..2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef ANALOG_CACHE_H_INCLUDED
#define ANALOG_CACHE_H_INCLUDED

#define ANALOG_CACHE_TTL	2000

#include "datastore.h"
#include "list.h"

struct ac_elem {
	unsigned long long ts;
	short value;
	unsigned char defined;
	unsigned char is_head;
	struct list_head list;
};

class AnalogCache;

struct acache_list_head {
	AnalogCache *cache;
	struct list_head list;
};

class AnalogCache {
private:
	unsigned long long quantum;
	int vquantum;
	struct ac_elem head;
	unsigned int physical;
	int need_update;
	int want_optimize;
	unsigned int ttl;
	unsigned long count;
	unsigned int add_counter;
	int touched;

	int ge(struct ac_elem *a, struct ac_elem *b);
public:
	struct acache_list_head cache_list;

	AnalogCache(unsigned int phys);

	struct ac_elem *get_last()
	{
		return list_entry(list_prev(&head.list), struct ac_elem, list);
	}

	unsigned long long begin_ts()
	{
		return list_entry(list_next(&head.list), struct ac_elem,
				list)->ts;
	}

	unsigned long long end_ts()
	{
		return list_entry(list_prev(&head.list), struct ac_elem,
				list)->ts;
	}

	int is_touched()
	{
		return touched;
	}

	void touch()
	{
		touched = 1;
	}

	void untouch()
	{
		touched = 0;
	}

	void set_quantum(unsigned long long q);
	void set_vquantum(int vq);

	void optimize();

	void update_all();
	void update();
	void update_cache(unsigned long long ts, short value);

	void invalidate();
	void invalidate_range(unsigned long long from, unsigned long long to);
	void cleanup(ac_elem *e);
	void clear();
	~AnalogCache();
};

class AnalogCacheControl {
private:
	struct list_head *caches;
	unsigned int count;
#ifdef CACHE_STATS
	unsigned int caches_count;
#endif

public:
	AnalogCacheControl(unsigned int n);

	void register_cache(unsigned int phys, AnalogCache *c);
	void unregister_cache(AnalogCache *c);

	void stats();

	void update_cache(unsigned int phys, unsigned long long ts,
			short value);

	void invalidate_all();
	void invalidate(unsigned int phys);
	void invalidate_range_all(unsigned long long from,
			unsigned long long to);
	void invalidate_range(unsigned int phys, unsigned long long from,
			unsigned long long to);
	void clear();
	~AnalogCacheControl();
};

extern AnalogCacheControl *ACC;

#endif
