//
// C++ Interface: analoggraph
//
// Description: 
//
//
// Author: Szymon Acedanski <accek@podlesie.net>, (C) 2004
// Copyright (C) 2006, 2007  Krzysztof Mazur <krzysiek@podlesie.net>	
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <qwidget.h>

#if QT_VERSION >= 0x40000
#include <QMouseEvent>
#include <QWheelEvent>
#endif

#if QT_VERSION >= 0x40000
# include <Q3PointArray>
# define QPointArray Q3PointArray
#endif

#include "bufferedwidget.h"
#include "control.h"
#include "analogcache.h"

class AnalogGraph : public BufferedWidget {
	Q_OBJECT

private:
	class AnalogPlot {
	public:
		AnalogPlot *Next;
		int Physical;
		QString	Caption;
		QColor Color;
		float Min;
		float Max;
		float UnitValue;
		QString Unit;
		AnalogCache *c;
		prec_t prec;
	};

	QString niceFormat(AnalogPlot *ap, float v);
	void draw_cursor(QPainter *p, int pos, QColor color);
	void calculate_cursor_value(struct cursor_state *cs, int x,
			unsigned long long oldt, int oldv, ac_elem *e);
	void paint_hgrid(QPainter *p, int w, int h);
	void paint_vgrid(QPainter *p, int w, int h,
			unsigned long long T0, unsigned long long T1);
	void draw_points(QPainter *p, QPen point_pen, QPointArray pts,
			unsigned int j);

	AnalogPlot *FirstPlot;
	AnalogPlot *LastPlot;

	QColor hgridColor;
	unsigned int hgridnLines;
	int *hgridLines;
	int cursor, cursor2;
	int mpressed;
	int old_pos;

protected:
	void paint(QPainter *p);
	void mousePressEvent(QMouseEvent *e);
	void mouseMoveEvent(QMouseEvent *e);
	void mouseReleaseEvent(QMouseEvent *e);
	void mouseDoubleClickEvent(QMouseEvent *e);
	void wheelEvent(QWheelEvent *e);

public:
	AnalogGraph(QWidget *parent = 0, const char *name = 0,
			Qt::WFlags f = 0);
	~AnalogGraph();

	void addPlot(int physical, QString caption, QColor color,
			float min, float max, float uv, QString unit,
			prec_t prec);
	void addHGrid(int nlines, int *lines, QColor color);
	void addSizeHint(int hint);

public slots:
	void update_timer();
};
