/*
 * AR-3c Monitor Protocol
 *
 * Copyright (C) 2004  Szymon Acedanski <accek@podlesie.net>
 * Copyright (C) 2005..2012  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef AR3CM_PROTO_H_INCLUDED
#define AR3CM_PROTO_H_INCLUDED

#define AR3CM_PROTO_VERSION	0x20401

#if defined(HAVE_INTTYPES_H)
#include <inttypes.h>
#elif defined(HAVE_STDINT_H)
#include <stdint.h>
#else
typedef unsigned long long __uint64_t;
typedef long long __int64_t;

typedef unsigned long __uint32_t;
typedef long __int32_t;

typedef unsigned short __uint16_t;
typedef signed short __int16_t;

typedef unsigned char __uint8_t;
typedef signed char __int8_t;
#endif

#include <sys/types.h>

#ifdef __GNUC__
#define AR3CM_PACK __attribute__((packed))
#define AR3CM_PACKA2 __attribute__((packed, aligned(2)))
#define AR3CM_PACKA4 __attribute__((packed, aligned(4)))
#define AR3CM_PACKA8 __attribute__((packed, aligned(8)))
#else
#pragma pack(1)
#define AR3CM_PACK
#define AR3CM_PACKA2
#define AR3CM_PACKA4
#define AR3CM_PACKA8
#endif

#ifdef HAVE_BYTESWAP_H
#include <byteswap.h>
#endif

#define __amp8_to_cpu(x)	(x)
#define __ampi8_to_cpu(x)	(x)
#define __cpu_to_amp8(x)	(x)
#define __cpu_to_ampi8(x)	(x)

#if BYTE_ORDER == LITTLE_ENDIAN
#define __amp16_to_cpu(x)	(x)
#define __ampi16_to_cpu(x)	(x)
#define __amp32_to_cpu(x)	(x)
#define __ampi32_to_cpu(x)	(x)
#define __amp64_to_cpu(x)	(x)
#define __ampi64_to_cpu(x)	(x)
#define __cpu_to_amp16(x)	(x)
#define __cpu_to_ampi16(x)	(x)
#define __cpu_to_amp32(x)	(x)
#define __cpu_to_ampi32(x)	(x)
#define __cpu_to_amp64(x)	(x)
#define __cpu_to_ampi64(x)	(x)
#elif BYTE_ORDER == BIG_ENDIAN
#define __amp16_to_cpu(x)	(bswap_16(x))
#define __ampi16_to_cpu(x)	((__int16_t) bswap_16(x))
#define __amp32_to_cpu(x)	(bswap_32(x))
#define __ampi32_to_cpu(x)	((__int32_t) bswap_32(x))
#define __amp64_to_cpu(x)	(bswap_64(x))
#define __ampi64_to_cpu(x)	((__int64_t) bswap_64(x))
#define __cpu_to_amp16(x)	(bswap_16(x))
#define __cpu_to_ampi16(x)	((__int16_t) bswap_16(x))
#define __cpu_to_amp32(x)	(bswap_32(x))
#define __cpu_to_ampi32(x)	((__int32_t) bswap_32(x))
#define __cpu_to_amp64(x)	(bswap_64(x))
#define __cpu_to_ampi64(x)	((__int64_t) bswap_64(x))
#else
#error "Please define ar3cmonitor protocol conversion macros"
#endif

struct amp_float_s {
	__uint32_t x;
} AR3CM_PACK;

struct amp64_s {
	__uint64_t v;
} AR3CM_PACK;

struct ampi64_s {
	__int64_t v;
} AR3CM_PACK;

struct amp32_s {
	__uint32_t v;
} AR3CM_PACK;

struct ampi32_s {
	__int32_t v;
} AR3CM_PACK;

struct amp16_s {
	__uint16_t v;
} AR3CM_PACK;

struct ampi16_s {
	__int16_t v;
} AR3CM_PACK;

struct amp8_s {
	__uint8_t v;
} AR3CM_PACK;

struct ampi8_s {
	__int8_t v;
} AR3CM_PACK;

typedef struct amp_float_s amp_float;
typedef struct amp64_s amp64_t;
typedef struct ampi64_s ampi64_t;
typedef struct amp32_s amp32_t;
typedef struct ampi32_s ampi32_t;
typedef struct amp16_s amp16_t;
typedef struct ampi16_s ampi16_t;
typedef struct amp8_s amp8_t;
typedef struct ampi8_s ampi8_t;
typedef float amp_float_cpu;

#define AMP_FLOAT(x) ((struct amp_float_s) { (x) })
#define AMP8(x) ((struct amp8_s) { (__uint8_t)(x) })
#define AMPI8(x) ((struct ampi8_s) { (__int8_t)(x) })
#define AMP16(x) ((struct amp16_s) { (__uint16_t)(x) })
#define AMPI16(x) ((struct ampi16_s) { (__int16_t)(x) })
#define AMP32(x) ((struct amp32_s) { (__uint32_t)(x) })
#define AMPI32(x) ((struct ampi32_s) { (__int32_t)(x) })
#define AMP64(x) ((struct amp64_s) { (__uint64_t)(x) })
#define AMPI64(x) ((struct ampi64_s) { (__int64_t)(x) })

#define amp8_to_cpu(x) __amp8_to_cpu((x).v)
#define ampi8_to_cpu(x) __ampi8_to_cpu((x).v)
#define amp16_to_cpu(x) __amp16_to_cpu((x).v)
#define ampi16_to_cpu(x) __ampi16_to_cpu((x).v)
#define amp32_to_cpu(x) __amp32_to_cpu((x).v)
#define ampi32_to_cpu(x) __ampi32_to_cpu((x).v)
#define amp64_to_cpu(x) __amp64_to_cpu((x).v)
#define ampi64_to_cpu(x) __ampi64_to_cpu((x).v)

#define cpu_to_amp8(x) AMP8(__cpu_to_amp8(x))
#define cpu_to_ampi8(x) AMPI8(__cpu_to_ampi8(x))
#define cpu_to_amp16(x) AMP16(__cpu_to_amp16(x))
#define cpu_to_ampi16(x) AMPI16(__cpu_to_ampi16(x))
#define cpu_to_amp32(x) AMP32(__cpu_to_amp32(x))
#define cpu_to_ampi32(x) AMPI32(__cpu_to_ampi32(x))
#define cpu_to_amp64(x) AMP64(__cpu_to_amp64(x))
#define cpu_to_ampi64(x) AMPI64(__cpu_to_ampi64(x))

static inline amp_float cpu_to_amp_float(amp_float_cpu x)
{
	union {
		amp_float_cpu f;
		amp_float i;
	} tmp;

	tmp.f = x;
	return AMP_FLOAT(__cpu_to_amp32(tmp.i.x));
}

static inline amp_float_cpu amp_float_to_cpu(amp_float x)
{
	union {
		amp_float_cpu f;
		amp_float i;
	} tmp;

	tmp.i.x = __amp32_to_cpu(x.x);
	return tmp.f;
}

struct ar3cm_proto_hdr {
	amp16_t type;
	amp16_t size;
} AR3CM_PACK;

struct ar3cm_proto_dblock {
       amp16_t offset;
       amp16_t size;
} AR3CM_PACK;

static __inline__ int ar3cm_dblock_access_ok(struct ar3cm_proto_dblock *b,
		size_t size)
{
	return ((size_t) amp16_to_cpu(b->offset)
			+ (size_t) amp16_to_cpu(b->size) <= size);
}

static __inline__ void *ar3cm_dblock_ptr(struct ar3cm_proto_dblock *b,
		void *base)
{
	return (char *)base + amp16_to_cpu(b->offset);
}

static __inline__ size_t ar3cm_dblock_size(struct ar3cm_proto_dblock *b)
{
	return amp16_to_cpu(b->size);
}

static __inline__ void ar3cm_dblock_set(struct ar3cm_proto_dblock *b,
		__uint16_t offset, __uint16_t size)
{
	b->offset = cpu_to_amp16(offset);
	b->size = cpu_to_amp16(size);
}

struct amp_string {
	amp16_t offset;	/* string offset in packet */
} AR3CM_PACK;

#define AMP_STRING(x)		((struct amp_string) { cpu_to_amp16((x)) })
#define AMP_STRING_INVALID	AMP_STRING(0)
typedef struct amp_string amp_string_t;

struct amp_string_cpu {
	__uint16_t offset;	/* string offset in packet */
} AR3CM_PACK;

#define AMP_STRING_CPU(x)	((struct amp_string_cpu) { (x) })
#define AMP_STRING_CPU_INVALID	AMP_STRING_CPU(0)

#define cpu_to_amp_string(x)		AMP_STRING(cpu_to_amp16((x)))
#define amp_string_to_cpu(x)		AMP_STRING_CPU(amp16_to_cpu((x)))

#define AR3CM_PROTO_ANALOGS		1
#define AR3CM_PROTO_RECORDDETECT	2
#define AR3CM_PROTO_BINARIES		3
#define AR3CM_PROTO_BINARYLOW		4
#define AR3CM_PROTO_BINARYHIGH		5
#define AR3CM_PROTO_VISSETUPSTART	6
#define AR3CM_PROTO_ANALGRAPHSTART	8
#define AR3CM_PROTO_ANALPLOT		9
#define AR3CM_PROTO_ANALGRAPHEND	10
#define AR3CM_PROTO_BINITEM		14
#define AR3CM_PROTO_VISSETUPEND		17
#define AR3CM_PROTO_NOP			18
#define AR3CM_PROTO_VMETER		21
#define AR3CM_PROTO_TITLE		22
#define AR3CM_PROTO_BINGRAPH		23
#define AR3CM_PROTO_ANALHGRID		24
#define AR3CM_PROTO_WIDGETSTYLE		25
#define AR3CM_PROTO_NEWSETUPSTART	26
#define AR3CM_PROTO_NEWTAB		27
#define AR3CM_PROTO_LEFTBOX		28
#define AR3CM_PROTO_BOTTOMBOX		29
#define AR3CM_PROTO_LABEL		30
#define AR3CM_PROTO_HBOX		32
#define AR3CM_PROTO_VBOX		33
#define AR3CM_PROTO_HLAYOUT		34
#define AR3CM_PROTO_VLAYOUT		35
#define AR3CM_PROTO_ENDLAYOUT		36
#define AR3CM_PROTO_SPACER		37
#define AR3CM_PROTO_SIZEHINT		38
#define AR3CM_PROTO_TIMESCALE		39
#define AR3CM_PROTO_STRETCH		40
#define AR3CM_PROTO_SEPARATOR		41
#define AR3CM_PROTO_BUTTON		42
#define AR3CM_PROTO_HIDDENTAB		43
#define AR3CM_PROTO_BIGLABEL		44
#define AR3CM_PROTO_SENDBINARIES	45
#define AR3CM_PROTO_STOREDA		46
#define AR3CM_PROTO_ENDOFDATA		47
#define AR3CM_PROTO_GETMINUTE		48
#define AR3CM_PROTO_STOREDBHIGH		49
#define AR3CM_PROTO_STOREDBLOW		50
#define AR3CM_PROTO_REJDERR		51
#define AR3CM_PROTO_MINNOTFOUND		52
#define AR3CM_PROTO_MINSTART		53
#define AR3CM_PROTO_SZYBOWSKAZ		54
#define AR3CM_PROTO_MASATEGOSYFU	55
#define AR3CM_PROTO_UNDEF		56
#define AR3CM_PROTO_ANALOG		57
#define AR3CM_PROTO_STOREDSINGLEA	58
#define AR3CM_PROTO_AMETER		59
#define AR3CM_PROTO_DMETER		60
#define AR3CM_PROTO_LEVEL		61	/* analog level */
#define AR3CM_PROTO_MINWIDTH		62
#define AR3CM_PROTO_MAXWIDTH		63
#define AR3CM_PROTO_MINHEIGHT		64
#define AR3CM_PROTO_MAXHEIGHT		65
#define AR3CM_PROTO_STRETCHA		66
#define AR3CM_PROTO_VERSION_PKT		67
#define AR3CM_PROTO_SCHEME_ID		68
#define AR3CM_PROTO_SETUP_INIT		69
#define AR3CM_PROTO_ENCODING		70
#define AR3CM_PROTO_SETLOCALE		71
#define AR3CM_PROTO_STATUS		72
#define AR3CM_PROTO_ORIENTATION		73
#define AR3CM_PROTO_GETTOKEN		74
#define AR3CM_PROTO_TOKEN		75
#define AR3CM_PROTO_SET_TIME		76
#define AR3CM_PROTO_PENDRIVE_POLL	77
#define AR3CM_PROTO_PENDRIVE_COPY	78
#define AR3CM_PROTO_PENDRIVE_NOTIFY	79
#define AR3CM_PROTO_CLOCK		80
#define AR3CM_PROTO_SCREENSAVERBOX	81
#define AR3CM_PROTO_SETCOLOR		82
#define AR3CM_PROTO_SETBGCOLOR		83
#define AR3CM_PROTO_EMPTY		84
#define AR3CM_PROTO_SET_OPT		85
#define AR3CM_PROTO_OBJECT		86
#define AR3CM_PROTO_LAYER_INFO		87
#define AR3CM_PROTO_TCN			88
#define AR3CM_PROTO_SET_FONT_SIZE	89
#define AR3CM_PROTO_SET_VALUE_FONT_SIZE	90
#define AR3CM_PROTO_SET_TIMESCALE_FONT_SIZE	91
#define AR3CM_PROTO_SET_SMALL_FONT_SIZE	92
#define AR3CM_PROTO_SET_PREC		93
#define AR3CM_PROTO_HIDE_IF		95
#define AR3CM_PROTO_HIDE_UNLESS		96

#define AR3CM_ORIENTATION_ROTATION	3
#define AR3CM_ORIENTATION_ROTN		0
#define AR3CM_ORIENTATION_ROTE		1
#define AR3CM_ORIENTATION_ROTS		2
#define AR3CM_ORIENTATION_ROTW		3

#define AR3CM_ORIENTATION_LEFT		4
#define AR3CM_ORIENTATION_DOWN		8

#define AR3CM_PROTO_SUCCESS		0
#define AR3CM_PROTO_EINVAL		22
#define AR3CM_PROTO_ENOSYS		38

struct ar3cm_proto_hsv {
	amp16_t h, s, v;
	amp16_t reserved;
} AR3CM_PACK;

struct ar3cm_proto_analogs {
	amp64_t timestamp;
	ampi16_t analogs[0];
} AR3CM_PACKA8;

struct ar3cm_proto_analog {
	amp64_t timestamp;
	ampi16_t number;
	ampi16_t value;
} AR3CM_PACKA4;

struct ar3cm_proto_binaries {
	amp16_t base;
	amp8_t data[0];
} AR3CM_PACKA2;

struct ar3cm_proto_recorddetect {
	amp16_t b_len;
	amp16_t a_len;
} AR3CM_PACK;

struct ar3cm_proto_binarylowhigh {
	amp16_t index;
	amp64_t timestamp;
} AR3CM_PACKA2;

struct ar3cm_proto_analgraph {
	char caption[0];
} AR3CM_PACK;

struct ar3cm_proto_analhgrid {
	struct ar3cm_proto_hsv color;
	amp8_t nlines;
	amp16_t lines[0];
} AR3CM_PACK;

#define AR3CM_ANALPLOT_ALONG	1

struct ar3cm_proto_analplot {
	amp16_t physical;
	amp_float min;
	amp_float max;
	amp_float unit_value;
	struct ar3cm_proto_hsv color;
	char unit[8];
	char caption[0];
	/* amp16_t flags; */
} AR3CM_PACK;

struct ar3cm_proto_newtab {
	char caption[0];
} AR3CM_PACK;

struct ar3cm_proto_binitem {
	amp16_t physical;
	struct ar3cm_proto_hsv ledcolor;
	struct ar3cm_proto_hsv activetextcolor;
	struct ar3cm_proto_hsv inactiveledcolor;
	amp8_t type;
	char caption[0];
} AR3CM_PACK;

#define AR3CM_BINTYPE_NORMAL 1
#define AR3CM_BINTYPE_DIAG   2

#define AR3CM_VMETER_ALONG	1

struct ar3cm_proto_vmeter {
	amp16_t physical;
	amp_float min;
	amp_float max;
	amp_float unit_value;
	struct ar3cm_proto_hsv color;
	char caption[0];
	/* amp16_t flags; */
} AR3CM_PACK;

struct ar3cm_proto_title {
	char title[0];
} AR3CM_PACK;

struct ar3cm_proto_bingraph {
	amp16_t physical;
	struct ar3cm_proto_hsv activecolor;
	struct ar3cm_proto_hsv inactivecolor;
	amp8_t type;
	char symbol[3];
} AR3CM_PACK;

#define AR3CM_ACTION_QUIT          1
#define AR3CM_ACTION_NOP           2
#define AR3CM_ACTION_DEBUG         3
#define AR3CM_ACTION_CHTAB         4
#define AR3CM_ACTION_OFFLINE       5
#define AR3CM_ACTION_REVIEW        6
#define AR3CM_ACTION_NORMAL        7
#define AR3CM_ACTION_DISCONNECT    8
#define AR3CM_ACTION_CONNECT       9

struct ar3cm_proto_action {
	amp16_t type;
} AR3CM_PACK;

struct ar3cm_proto_button {
	char label[0];
} AR3CM_PACK;

struct ar3cm_proto_label {
	char label[0];
} AR3CM_PACK;

struct ar3cm_proto_debug {
	char info[0];
} AR3CM_PACK;

struct ar3cm_proto_chtab {
	amp16_t tab;
} AR3CM_PACK;

struct ar3cm_proto_getminute {
	amp32_t min;
	ampi32_t layer;
} AR3CM_PACK;

struct ar3cm_level {
	struct ar3cm_proto_hsv color;
	amp_float level;
	char caption[16];
} AR3CM_PACK;

struct ar3cm_proto_szybowskaz {
	amp16_t posphys;
	amp16_t velphys;
	amp_float minpos;
	amp_float maxpos;
	amp_float posuv;
	amp_float minvel;
	amp_float maxvel;
	amp_float veluv;
	amp32_t level_count;
	struct ar3cm_level level[0];
} AR3CM_PACK;

#define AR3CM_WSKAZ_EXT_DUAL		0x01
#define AR3CM_WSKAZ_EXT_DUALEMU		0x02

struct ar3cm_proto_szybowskaz_ext {
	amp32_t flags;
	amp16_t style;
	struct ar3cm_proto_hsv color;
	struct ar3cm_proto_hsv color2;
	union {
		struct {
			amp16_t pos2phys;
			amp_float pos2uv;
		};
		amp_float pos2offset;
	};
} AR3CM_PACK;

struct ar3cm_proto_masategosyfu {
	amp64_t calk_start;
	amp_float akt_masa;
	amp_float calk_masa;
} AR3CM_PACK;

struct ar3cm_proto_undef {
	amp64_t ts;
	amp32_t flags;
} AR3CM_PACK;

#define AR3CM_AMETER_UNIPOLAR	1
#define AR3CM_AMETER_VALUE	2
#define AR3CM_AMETER_ALONG	4

struct ar3cm_proto_ameter {
	amp16_t physical;
	amp16_t flags;
	ampi16_t angle;
	ampi16_t scale;
	amp_float min;
	amp_float max;
	amp_float unit_value;
	struct ar3cm_proto_hsv color;
	char unit[16];
	char caption[0];
} AR3CM_PACK;

#define AR3CM_DMETER_SHOW_UNIT		1
#define AR3CM_DMETER_ALONG		2

struct ar3cm_proto_dmeter {
	amp16_t physical;
	amp16_t flags;
	amp_float unit_value;
	struct ar3cm_proto_hsv color;
	char unit[16];
	amp_string_t prefix;
} AR3CM_PACK;

struct ar3cm_proto_clock {
	amp32_t flags;
	char format[];
} AR3CM_PACK;

struct ar3cm_proto_object {
	amp32_t flags;
	char command[];
} AR3CM_PACK;

struct ar3cm_proto_alevel {
	amp_float value;
	struct ar3cm_proto_hsv color;
	char caption[32];
} AR3CM_PACK;

#define AR3CM_BINGRAPH_NORMAL  1
#define AR3CM_BINGRAPH_DZWONEK 2

#define AR3CM_TOKEN_NONE	0
#define AR3CM_TOKEN_SHA1	1

#define AR3CM_TTL_ONETIME	0

struct ar3cm_proto_gettoken {
	amp32_t types;
	amp32_t flags;
	amp64_t ttl;
} AR3CM_PACK;

struct ar3cm_proto_token {
	amp64_t ttl;
	amp32_t type;
	amp32_t flags;
	struct ar3cm_proto_dblock token;
} AR3CM_PACK;

struct ar3cm_proto_signature {
	struct ar3cm_proto_dblock sign;
} AR3CM_PACK;

#define AR3CM_PROTO_MAY_ADJUST	1

struct ar3cm_proto_set_time {
	amp64_t ts;
	amp32_t flags;
	struct ar3cm_proto_signature signature;
} AR3CM_PACK;

#define AR3CM_PROTO_PENDRIVE_POLL_ERR		1
#define AR3CM_PROTO_PENDRIVE_POLL_INSERT	2
#define AR3CM_PROTO_PENDRIVE_POLL_EJECT		4
#define AR3CM_PROTO_PENDRIVE_POLL_COMPLETE	8
#define AR3CM_PROTO_PENDRIVE_POLL_PROGRESS	16

struct ar3cm_proto_pendrive_poll {
	amp32_t events;
	struct ar3cm_proto_signature signature;
} AR3CM_PACK;

#define AR3CM_PROTO_PENDRIVE_FORMAT_LOGGER2	2
#define AR3CM_PROTO_PENDRIVE_FORMAT_LOGGER3	3
#define AR3CM_PROTO_PENDRIVE_FORMAT_LOGGER4	4

struct ar3cm_proto_pendrive_copy {
	amp64_t from;
	amp64_t to;
	amp16_t format;
	struct ar3cm_proto_signature signature;
} AR3CM_PACK;

struct ar3cm_proto_pendrive_notify {
	amp32_t events;
	amp32_t status;
} AR3CM_PACK;

struct ar3cm_proto_setcolor {
	struct ar3cm_proto_hsv color;
} AR3CM_PACK;

struct ar3cm_proto_set_opt {
	amp32_t opt;
	amp32_t value;
} AR3CM_PACK;

#define AR3CM_PROTO_LAYER_EXIST		1
#define AR3CM_PROTO_LAYER_ACTIVE	2
#define AR3CM_PROTO_LAYER_RELATIVE	4
struct ar3cm_proto_layer_info {
	amp64_t begin;
	amp64_t end;
	amp32_t flags;
	ampi32_t id;
} AR3CM_PACK;

struct ar3cm_proto_tcn {
	amp64_t old_ts;
	amp64_t new_ts;
	amp32_t tcn_seq;
	amp32_t tcn_source;
} AR3CM_PACK;

#define AR3CM_PROTO_OPT_LED_NO_SYMBOL	1

#define AR3CM_PROTO_FONT_SIZE_DEFAULT	0x00000000
#define AR3CM_PROTO_FONT_SIZE_PX	0x01000000
#define AR3CM_PROTO_FONT_SIZE_MAGNIFY	0x02000000
#define AR3CM_PROTO_FONT_SIZE_PLUS	0x03000000
#define AR3CM_PROTO_FONT_SIZE_MASK	0xff000000

#define AR3CM_PROTO_PREC_SDIGITS	0x01000000
#define AR3CM_PROTO_PREC_DDIGITS	0x02000000
#define AR3CM_PROTO_PREC_LZERO		0x80000000 /* show leading zeros */
#define AR3CM_PROTO_PREC_MASK		0xff000000

#endif
