//
// C++ Interface: ar3cmonitor
//
// Description: 
//
//
// Author: Szymon Acedanski <accek@podlesie.net>, (C) 2004
// Copyright (C) 2005..2008  Krzysztof Mazur <krzysiek@podlesie.net>
// Copyright (C) 2008  Krzysztof Kasperski <casper@podlesie.net>
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef AR3CMONITOR_H_INCLUDED
#define AR3CMONITOR_H_INCLUDED

#include <qwidget.h>
#include <qpushbutton.h>
#include <qapplication.h>

#if QT_VERSION >= 0x40000
# include <Q3PopupMenu>

# define QPopupMenu	Q3PopupMenu
#endif

#include "consts.h"

class QVBoxLayout;
class QHBoxLayout;
class QBoxLayout;
class QLabel;
class QTimer;
class QKeyEvent;
class MyTabWidget;
class AnalogGraph;
class Action;
class MainStatusBar;
class DzwonekGraph;

extern QApplication *qa;
extern int show_points_count;

#define AR3CM_PAGE_MAIN	0
#define AR3CM_PAGE_ANALOGS 0
#define AR3CM_PAGE_BINARY  1

class Ar3cMonitor : public QWidget {
	Q_OBJECT

private:
	QVBoxLayout *RootLayout;
	QVBoxLayout *Layout;
	MyTabWidget *Tabs;
	QLabel *StatusLabel;
	QLabel *ClockLabel;
	QLabel *TitleLabel;
	QHBoxLayout *HLayout;		/* the main horizontal layout */
	QVBoxLayout *VLayout;		/* the inner vertical layout */
	QWidget *VLWidget;
	MainStatusBar *StatusBar;
	QPushButton *jmp_button;
	QWidget *BodyWidget;
	QVBoxLayout *BodyLayout;

	QPopupMenu *options;
	int swapcur0_id;
	int swapcur1_id;
	int swapcur2_id;

	QBoxLayout *CurrentLayout;
	QWidget *CurrentWidget;
	AnalogGraph *CurrentAnalogGraph;
	QWidget *CurrentHiddenTab;
	DzwonekGraph *CurrentDzwonek;
	QWidget *TabsTbl[AR3CMON_MAX_TABS];
	QString TabsCaptions[AR3CMON_MAX_TABS];
	
	QWidget *AttrWidget;
	int TabsCnt;

	class LayoutsStack {
	public:
		LayoutsStack *Next;
		QWidget *CurrentWidget;
		QBoxLayout *CurrentLayout;
		QColor bgcolor;
		QColor color;
	};

	LayoutsStack *CurrentStack;

	int tabIndex;

	void createStatusLabel(QString text);
	void setupTitleBar();
	void closeEvent(QCloseEvent *q);
	void configure_menu();

private slots:
	void currentPush();
	void newTimestamp();
	void tabChanged();
	void commOfflineConnected();
	void commConnected();
	void commConnecting();
	void commError();
	void jump_clicked();
	void setTime_clicked();
	void usb_clicked();
	void calibrate_clicked();
	void about();
	void swap_cursor0();
	void swap_cursor1();
	void swap_cursor2();
	void increase_timerange();
	void decrease_timerange();
	void timerange_15s();
	void timerange_30s();
	void timerange_1m();
	void timerange_2m();
	void timerange_5m();
	void timerange_10m();
	void timerange_15m();
	void timerange_30m();
	void timerange_1h();
	void timerange_2h();
	void timerange_3h();
	void select_active_layer();
	void select_0_layer();
	void select_1_layer();
	void select_2_layer();
	void select_3_layer();
	void select_4_layer();
	void select_5_layer();
	void select_6_layer();
	void select_7_layer();
	
	void menu_quit();
	void set_hostname();

protected:
	virtual void keyPressEvent(QKeyEvent* e);

public:
	QWidget *RootWidget;

	Ar3cMonitor(QWidget *parent = 0, const char *name = 0,
			Qt::WFlags f = 0);

	static QString appName()
	{
		return tr("ar3cmonitor");
	}

	void interface_reinit();
	void setTitle(QString title);
	void newTab(QString caption, bool hidden);
	void hideHiddenTab();
	void activateTab(int index);
	void newLeftbox();
	void newScreenSaver();
	void newBottombox();
	void newLabel(QString text);
	void newEmpty();
	void newObject(const char *command);
	void newTClockLabel(const char *format);
	void newBigLabel(QString text);
	void newHBox();
	void newVBox();
	void newHLayout();
	void newVLayout();
	void endLayout();
	void setBgColor(QColor bgcolor);
	void setColor(QColor color);
	QColor getBgColor() const;
	QColor getColor() const;
	void newSeparator();
	void newSpacer();
	void newStretch();
	void addLed(int physical, QString caption, QColor ledcolor,
			QColor activetextcolor, QColor ledinactivecolor,
			bool diag);
	void newGraph();
	void newTimeScale();
	void setSizeHint(int hint);
	void setMinWidthAttr(int hint);
	void setMaxWidthAttr(int hint);
	void setMinHeightAttr(int hint);
	void setMaxHeightAttr(int hint);
	void setStretchAttr(int hint);
	void hide_cond(int physical, int not_cond);
	void addPlot(int physical, QString caption, QColor color,
			float min, float max, float uv, QString unit);
	void addHGrid(int nlines, int* lines, QColor color);
	void newDzwonek(int physical, QString symbol, QColor activecolor,
			QColor inactivecolor);
	void newBinGraph(int physical, QString symbol, QColor activecolor,
			QColor inactivecolor);
	void newVMeter(int physical, QString caption, QColor color,
			float min, float max, float uv, unsigned int flags);
	void newAMeter(int physical, QString caption, QColor color,
			float min, float max, float uv, int angle,
			int scale, QString unit, unsigned long flags);
	void newDMeter(int physical, QColor color, float uv,
			QString unit, unsigned long flags,
			QString prefix);
	void newButton(QString caption, Action* action);
	void newSzybowskaz(int posPhys, int velPhys, float posUV,
			float velUV, float posMin, float posMax, float velMin,
			float velMax, int level_count,
			struct ar3cm_level *levels, int left);
	void visSetupEnd();

	void handle_key(QKeyEvent *e);
};

extern Ar3cMonitor* MainWindow;

static inline unsigned long long calculateT(int x, unsigned long long limit,
		int w, unsigned long long timerange)
{
	/* przy x < 4096 timerange powinno byc mniejsze jak 26 godziny */
	if (!w)
		return 0ULL;
	return limit + (timerange * x) / w;
}

static inline long long calculateX(unsigned long long timestamp,
		unsigned long long limit, int w, unsigned long long timerange)
{
	timerange >>= 14;
	limit >>= 14;
	timestamp >>= 14;
	unsigned long long a = timestamp * w / timerange;
	unsigned long long b = limit * w / timerange;
	return (long long) a - (long long) b;
}

static inline int calculateX_bin(unsigned long long timestamp,
		unsigned long long limit, int w, unsigned long long timerange)
{
	long long x = calculateX(timestamp, limit, w, timerange);

	if (x < 0)
		x = 0;
	else if (x > w)
		x = w;
	return x;
}

#if QT_VERSION >= 0x40000
# define ARMON_COLOR(x)		(Qt::x)
#else
# define ARMON_COLOR(x)		(x)
#endif

#if QT_VERSION >= 0x40000
#define QT_STRONG_FOCUS Qt::StrongFocus
#else
#define QT_STRONG_FOCUS QWidget::StrongFocus
#endif

#include <stdio.h>

static inline size_t new_tsize(size_t size)
{
	return ((size + AR3CMON_NEW_ALIGNMENT - 1) & AR3CMON_NEW_ALIGNMENT_MASK)
		+ AR3CMON_NEW_OVERHEAD;
}

static inline size_t monalloc_tsize(size_t size)
{
	return ((size + MONALLOC_ALIGNMENT - 1) & MONALLOC_ALIGNMENT_MASK)
		+ MONALLOC_OVERHEAD;
}

#define TS_MS(ts) ((ts) / 1000000LL)
#define TS_SEC(ts) ((ts) / 1000000000LL)
#define TS_MIN(ts) ((ts) / 60000000000LL)
#define TS_HOUR(ts) ((ts) / 3600000000000LL)

#define MS_TS(ts) ((ts) * 1000000LL)
#define SEC_TS(ts) ((ts) * 1000000000LL)
#define MIN_TS(ts) ((ts) * 60000000000LL)
#define HOUR_TS(ts) ((ts) * 3600000000000LL)

#define BUG()								\
	do {								\
		fprintf(stderr, "BUG: at %s:%d\n", __FILE__, __LINE__);	\
		abort();						\
	} while (0)

#define BUG_ON(cond)							\
	do {								\
		if (cond)						\
			BUG();						\
	} while (0)

#define min(a,b) ((a) < (b) ? (a) : (b))
#define max(a,b) ((a) > (b) ? (a) : (b))

#ifndef HAVE_LOCALTIME_R
#include <time.h>

static struct tm *localtime_r(const time_t *timep, struct tm *result)
	__attribute__((unused));

static struct tm *localtime_r(const time_t *timep, struct tm *result)
{
	struct tm *tm;

	tm = localtime(timep);
	if (tm == NULL)
		return tm;
	memcpy(result, tm, sizeof(struct tm));
	return result;
}
#endif

#ifndef HAVE_STRNDUP
#include <string.h>

static char *strndup(const char *s, size_t n) __attribute__((unused));

static char *strndup(const char *s, size_t n)
{
	char *dst = (char *) malloc(n + 1);
	char *t = dst;

	if (dst != NULL) {
		while ((*s != '\0') && n--)
			*(t++) = *(s++);
		*t = '\0';
	}
	return dst;
}
#endif

QString text2unicode(QString a);

#define get_unaligned(ptr)						\
	({								\
		struct __str {						\
			typeof(*(ptr)) __v;				\
	 	} __attribute__((packed)) *__p = (__str *) (ptr);	\
		__p->__v;						\
	})

#define put_unaligned(val, ptr)						\
	do {								\
		struct __str {						\
			typeof(*(ptr)) __v;				\
		} __attribute__((packed)) *__p = (__str *) (ptr);	\
		__p->__v = (val);					\
	} while(0)

#ifndef DATA_DIR
#define DATA_DIR ""
#endif

#endif
