/*
 * ar3cmonitor - cdata.cpp
 *
 * Copyright (C) 2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <cdata.h>
#include <errno.h>
#include <control.h>

/**
 * cdata_get_analog - get current value of analog @phys
 * @phys: analog physical number
 * @value: returned analog value
 *
 * This function returns 0 on success; negative error code otherwise.
 */
int cdata_get_analog(unsigned int phys, int *value)
{
	AList::AIterator it(CO->analog()->Cursor.scanLeftBefore(
				CO->cursorStamp()), phys);
	it.positionLeft();

	if (!CO->haveData())
		return -ENOENT;

	if (!it.valid() || !it.data()->defined())
		return -ENOENT;

	*value = it.get();
	return 0;
}

/**
 * cdata_get_along - get current value of long analog @phys
 * @phys: analog physical number
 * @value: returned analog value
 *
 * This function returns 0 on success; negative error code otherwise.
 */
int cdata_get_along(unsigned int phys, int *value)
{
	AList::AIterator it1(CO->analog()->Cursor.scanLeftBefore(CO->
			cursorStamp()), phys);
	AList::AIterator it2(CO->analog()->Cursor.scanLeftBefore(CO->
			cursorStamp()), phys + 1);

	it1.positionLeft();
	it2.positionLeft();

	if (!it1.valid() || !it1.data()->defined() ||
			!it2.valid() || !it2.data()->defined())
		return -ENOENT;

	*value = (((int) it2.get()) << 16) + (unsigned short) it1.get();
	return 0;
}

/**
 * cdata_get_binary - get current value of binary @phys
 * @phys: binary physical number
 * @value: returned binary value
 *
 * This function returns 0 on success; negative error code otherwise.
 */
int cdata_get_binary(unsigned int phys, int *value)
{
	TimestampedList<BRecord>::Iterator it =
		CO->binary(phys)->Cursor.scanLeftBefore(CO->cursorStamp());

	if (!it.valid())
		return -ENOENT;
	switch (it.data()->state) {
	case Undefined:
		return -ENOENT;
	case High:
		*value = 1;
		break;
	case Low:
		*value = 0;
		break;
	}
	return 0;
}
