/*
 * ar3cmonitor - wskaz.cpp
 *
 * Copyright (C) 2004  Szymon Acedanski <accek@podlesie.net>
 * Copyright (C) 2006..2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <qpainter.h>
#include <qlayout.h>

#include <cdata.h>
#include "wskaz.h"
#include "control.h"
#include "ar3cm_proto.h"

Wskaz::Wskaz(int _posPhys, int _velPhys, float _posUV,
		float _velUV, float _posMin, float _posMax,
		float _velMin, float _velMax, int _level_count,
		struct ar3cm_level *_levels, int left,
		QWidget *parent, const char *name,
		Qt::WFlags fl)
	: QWidget(parent, name, fl), posPhys(_posPhys), velPhys(_velPhys),
	posUV(_posUV), velUV(_velUV), posMin(_posMin), posMax(_posMax),
	velMin(_velMin), velMax(_velMax)
{
	QVBoxLayout *l1 = new QVBoxLayout(this);
	QGridLayout *topg = new QGridLayout(l1, 2, 3);
	topg->setColStretch(0, 0 /*1000 */ );
	QFont f(CO->valueFont());

	setBackgroundColor(Qt::black);
#if QT_VERSION >= 0x40100
	setAutoFillBackground(true);
#endif

	OnOffLabel *posu = new OnOffLabel(QString("m"), ARMON_COLOR(white),
			ARMON_COLOR(gray), this);
	posu->setFont(f);
	posu->setMinimumHeight(posu->fontMetrics().height());
	posu->setMinimumWidth(posu->fontMetrics().width(QString("m")));
	topg->addWidget(posu, 0, 2);

	f.setBold(TRUE);
	PosV = new OnOffLabel(QString("???"), ARMON_COLOR(white),
		ARMON_COLOR(white), this);
	PosV->setFont(f);
	PosV->setMinimumHeight(PosV->fontMetrics().height());
	PosV->setMinimumWidth(PosV->fontMetrics().width(QString("-9999.99"))
			* 6 / 5);
	topg->addWidget(PosV, 0, 1);

	QHBoxLayout *botl = new QHBoxLayout(l1);
	Bar = new WskazBar(posMin, posMax, left, this);
	Scale = new WskazScale(posMin, posMax, _level_count, _levels, this);
	if (left) {
		botl->addWidget(Scale);
		botl->addWidget(Bar);
	} else {
		botl->addWidget(Bar);
		botl->addWidget(Scale);
	}

	l1->setStretchFactor(topg, 0);
	l1->setStretchFactor(botl, 1000);

	connect(CO, SIGNAL(sigUpdateTimer()), SLOT(upd()));
	connect(CO, SIGNAL(sigPleaseUpdate()), SLOT(upd()));

	upd();
}

void Wskaz::upd()
{
	int value;

	if (cdata_get_along(posPhys, &value)) {
		PosV->setText(QString("???"));
		Bar->set_inval();
		return;
	}
	float pos = value / posUV;

	PosV->setText(QString().sprintf("%7.2f", pos));
	Bar->setValue(pos);
}

WskazBar::WskazBar(float _Min, float _Max, int _left, QWidget *parent,
		const char *name, Qt::WFlags f)
	: BufferedWidget(parent, name, f), Min(_Min), Max(_Max), left(_left)
{
	setMinimumWidth(30);
	setMaximumWidth(40);

#if QT_VERSION >= 0x40000
#elif QT_VERSION >= 0x30000
	setBackgroundMode(Qt::NoBackground);
#else
	setBackgroundMode(QWidget::NoBackground);
#endif

#if QT_VERSION >= 0x40100
	setAutoFillBackground(true);
#endif
	value = 0.0;
}

void WskazBar::paint(QPainter *p)
{
	int h = height();

	if (!valid)
		p->drawText(rect(), Qt::AlignCenter, QString("?"));
	else {
		QPen pen(p->pen());
		pen.setColor(QColor(180, 255, 255, QColor::Hsv));
		pen.setWidth(2);
		p->setPen(pen);
		int y = (int) (h - (value - Min) * h / (Max - Min));
		if (y > -h / 24) {
			if (y > h)
				p->drawLine(width() / 2, 0, width() / 2, h);
			else {
				if (left)
					p->drawLine(1, y, 2 * width() / 3, y);
				else
					p->drawLine(width() / 3, y, width(), y);
				p->drawLine(width() / 2, 0, width() / 2, y);

				p->setPen(Qt::NoPen);
				p->setBrush(QColor(180, 255, 255, QColor::Hsv));
				p->drawRect(width() / 3, y, width() / 3,
						h / 24);
			}
		}

		pen.setColor(QColor(0, 0, 170, QColor::Hsv));
		pen.setWidth(4);
		p->setPen(pen);
		p->drawLine(3, 0, 3, h);
		p->drawLine(width() - 3, 0, width() - 2, h);
	}

	p->setPen(ARMON_COLOR(white));
	p->drawLine(0, 0, width(), 0);

}

void WskazBar::setValue(float v)
{
	value = v;
	valid = 1;
	update();
}

void WskazBar::set_inval(void)
{
	valid = 0;
	update();
}

WskazScale::WskazScale(float _Min, float _Max, int _level_count,
		struct ar3cm_level *_levels, QWidget *parent,
		const char *name, Qt::WFlags f)
	: BufferedWidget(parent, name, f), Min(_Min), Max(_Max),
	level_count(_level_count)
{
	levels = (struct ar3cm_level *) malloc(sizeof(struct ar3cm_level)
		* _level_count);
	Q_CHECK_PTR(levels);
	memcpy(levels, _levels, sizeof(struct ar3cm_level) * _level_count);

	setBackgroundColor(Qt::black);
#if QT_VERSION >= 0x40100
	setAutoFillBackground(true);
#endif
}

void WskazScale::paint(QPainter *p)
{
	int h = height();
	int i;

	p->setPen(ARMON_COLOR(white));
	p->drawLine(0, 0, width(), 0);
	p->drawLine(0, 0, 0, height());
	int text_h = QFontMetrics(p->font()).height();

	for (i = 0; i < level_count; i++) {
		struct ar3cm_level *l = &levels[i];
		int y = (int) (h - (amp_float_to_cpu(l->level) - Min) * h
				/ (Max - Min));
		p->setPen(QColor(amp16_to_cpu(l->color.h),
					amp16_to_cpu(l->color.s),
					amp16_to_cpu(l->color.v),
				QColor::Hsv));
		p->drawLine(0, y, width(), y);


		p->drawText(5, y - text_h, width(), text_h, Qt::AlignCenter,
			QString(l->caption));
	}
}
